@@ -1,12 +1,22 @@
+from PyQt5.QtCore import pyqtSignal, QEvent, QObject
 from PyQt5.QtGui import QIcon, QPixmap
 from PyQt5.QtWidgets import QApplication, QMainWindow
 
 
-class WindowController:
+class WindowController(QObject):
     """ Wrapper class with methods for manipulating the main window. """
 
+    activated = pyqtSignal()  # Emitted when the window state changes from inactive to active.
+
     def __init__(self, w_window:QMainWindow) -> None:
+        super().__init__(w_window)
         self._w_window = w_window  # Main Qt window.
+        w_window.installEventFilter(self)
+
+    def eventFilter(self, _, event:QEvent) -> bool:
+        if event.type() == QEvent.WindowActivate:
+            self.activated.emit()
+        return False
 
     def show(self) -> None:
         """ Show the window, move it in front of other windows, and activate focus.
