@@ -11,18 +11,14 @@ from spectra_lexer.resource.translations import ExamplesDict, RuleID, Translatio
 class StenoAnalyzer:
     """ Key-converting wrapper for the lexer. Also uses multiprocessing to make an examples index. """
 
-    # Rule info strings for analysis results. The output is nowhere near reliable if some keys are unmatched.
-    INFO_COMPLETE = "Found complete match."
-    INFO_INCOMPLETE = "Incomplete match. Not reliable."
-    INFO_EMPTY = "No matches found."
-
     def __init__(self, converter:StenoKeyConverter, lexer:StenoLexer, factory:StenoRuleFactory,
-                 refmap:Mapping[LexerRule, StenoRule], idmap:Mapping[LexerRule, RuleID]) -> None:
+                 refmap:Mapping[LexerRule, StenoRule], idmap:Mapping[LexerRule, RuleID], rule_sep:StenoRule) -> None:
         self._converter = converter  # Converts between RTFCRE and s-keys formats.
         self._lexer = lexer          # Main analysis engine; operates only on s-keys.
         self._factory = factory      # Creates steno rules from analysis data.
         self._refmap = refmap        # Mapping of lexer rule objects to their original StenoRules.
         self._idmap = idmap          # Mapping of lexer rule objects to valid example rule IDs.
+        self._rule_sep = rule_sep    # Stroke separator rule. Used as a delimiter. Letters are not allowed.
 
     def _to_skeys(self, keys:str) -> str:
         """ Convert user RTFCRE steno <keys> to s-keys. """
@@ -44,15 +40,14 @@ class StenoAnalyzer:
             child = self._refmap[lr]
             length = len(lr.letters)
             self._factory.connect(child, start, length)
-        info = self.INFO_COMPLETE
         unmatched_skeys = result.unmatched_skeys
         if unmatched_skeys:
-            info = self.INFO_INCOMPLETE if result.rules else self.INFO_EMPTY
+            self._factory.push()
             unmatched_keys = self._to_rtfcre(unmatched_skeys)
-            nletters = len(letters)
-            self._factory.connect_unmatched(unmatched_keys, nletters, "unmatched keys")
+            child = self._factory.build(unmatched_keys, "", "unmatched keys", is_unmatched=True)
+            self._factory.connect_rest(child, len(letters))
         keys = self._to_rtfcre(skeys)
-        return self._factory.build(keys, letters, info)
+        return self._factory.build(keys, letters)
 
     def best_translation(self, keys_iter:Iterable[str], letters:str) -> str:
         """ Return the best (most accurate) match to <letters> out of <keys_iter> according to lexer ranking. """
@@ -66,6 +61,27 @@ class StenoAnalyzer:
             best_index = self._lexer.best_translation(skeys_list, letters)
         return keys_list[best_index]
 
+    def compound_query(self, translations:TranslationsIter) -> StenoRule:
+        """ Perform queries for several translations and combine the results.
+            Only translations with keys are analyzed and delimited by stroke separators. """
+        all_skeys = all_letters = ""
+        sep_skeys = self._to_skeys(self._rule_sep.keys)
+        self._factory.push()
+        is_first = True
+        for keys, letters in translations:
+            if keys:
+                offset = len(all_letters)
+                if not is_first:
+                    all_skeys += sep_skeys
+                    self._factory.connect(self._rule_sep, offset, 0)
+                all_skeys += self._to_skeys(keys)
+                rule = self.query(keys, letters)
+                self._factory.connect(rule, offset, len(letters))
+                is_first = False
+            all_letters += letters
+        all_keys = self._to_rtfcre(all_skeys)
+        return self._factory.build(all_keys, all_letters)
+
     def _query_rule_ids(self, keys:str, letters:str) -> List[str]:
         """ Make a parallel-safe lexer query and return the result as a list of strings.
             Results may be returned out of order, so the output starts with the original keys and letters.
@@ -91,31 +107,3 @@ class StenoAnalyzer:
             for r_id in rule_ids:
                 index[r_id][keys] = letters
         return index
-
-    def delimit(self, rules:Iterable[StenoRule], keys_delim:str, letters_delim="") -> List[StenoRule]:
-        """ Delimit a sequence of rules with <keys_delim> and <letters_delim>. """
-        self._factory.push()
-        rule_delim = self._factory.build(keys_delim, letters_delim, "delimiter")
-        delimited = []
-        for r in rules:
-            delimited += [r, rule_delim]
-        return delimited[:-1]
-
-    def join(self, rules:Iterable[StenoRule]) -> StenoRule:
-        """ Join several rules into one by concatenation. """
-        skeys = letters = ""
-        offset = 0
-        self._factory.push()
-        for r in rules:
-            skeys += self._to_skeys(r.keys)
-            letters += r.letters
-            length = len(r.letters)
-            self._factory.connect(r, offset, length)
-            offset += length
-        keys = self._to_rtfcre(skeys)
-        return self._factory.build(keys, letters, "combined rules")
-
-    def normalize_keys(self, keys:str) -> str:
-        """ Normalize a set of RTFCRE keys by converting back and forth. """
-        skeys = self._to_skeys(keys)
-        return self._to_rtfcre(skeys)
