@@ -1,48 +1,45 @@
 from typing import Dict, Tuple
 
 from spectra_lexer.resource.translations import ExamplesDict, RuleID, TranslationsDict
-from spectra_lexer.search.index import RegexError, StripCaseIndex
+from spectra_lexer.search.index import RegexError, StringKeyIndex, StripCaseIndex
 from spectra_lexer.search.multidict import forward_multidict, reverse_multidict
 
-MatchDict = Dict[str, Tuple[str, ...]]         # JSON-compatible dict of search results.
-SearchData = Tuple[MatchDict, StripCaseIndex]  # Key search index paired with a standard dictionary for value lookup.
+MatchTuple = Tuple[str, ...]                   # JSON-compatible sequence of search results.
+MatchDict = Dict[str, MatchTuple]              # JSON-compatible dict of search results.
+SearchData = Tuple[MatchDict, StringKeyIndex]  # Key search index paired with a standard dictionary for value lookup.
 
-INDEX_DELIM = ";;"                 # Delimiter between rule ID and query for example searches. Mostly arbitrary.
-EXPAND_KEY = '[more...]'           # If present, repeating the search with a higher <count> will return more items.
+# Reserved sentinel keys in every search dict. These (and only these) map to an empty tuple of values.
+EXPAND_KEY = '[more...]'           # If present, repeating the search with a higher count will return more items.
 BAD_REGEX_KEY = '[INVALID REGEX]'  # If present, the search input could not be compiled as a regular expression.
+_SENTINEL_MAP = {k: () for k in (EXPAND_KEY, BAD_REGEX_KEY)}
+_EMPTY_DATA = (_SENTINEL_MAP, StringKeyIndex())
 
-# Reserved string keys in every lookup dict. These (and only these) map to an empty tuple of values.
-SENTINEL_KEYS = [EXPAND_KEY, BAD_REGEX_KEY]
-
-
-def _to_search_data(d:MatchDict, strip_chars=" ") -> SearchData:
-    """ Build a new string search index that strips characters and removes case. """
-    index = StripCaseIndex(d, strip_chars)
-    for k in SENTINEL_KEYS:
-        d[k] = ()
-    return d, index
-
-
-_EMPTY_DATA = _to_search_data({})
+INDEX_DELIM = ';;'  # Delimiter between rule ID and query for example searches. Mostly arbitrary.
 
 
 class SearchEngine:
     """ A hybrid forward+reverse steno translation search engine with support for rule example lookup. """
 
-    def __init__(self, strip_chars:str) -> None:
-        self._strip_chars = strip_chars  # Characters to remove during search.
-        self._tr_strokes = _EMPTY_DATA   # Forward translation search data (strokes -> text).
-        self._tr_text = _EMPTY_DATA      # Reverse translation search data (text -> strokes).
-        self._examples_raw = {}          # Contains steno rule IDs mapped to dicts of example translations.
-        self._examples_cache = {}        # Cache of example search data for each rule ID and mode.
+    def __init__(self, strip_strokes:str, strip_text:str) -> None:
+        self._strip_strokes = strip_strokes  # Characters to ignore during stroke search.
+        self._strip_text = strip_text        # Characters to ignore during text search.
+        self._tr_strokes = _EMPTY_DATA       # Forward translation search data (strokes -> text).
+        self._tr_text = _EMPTY_DATA          # Reverse translation search data (text -> strokes).
+        self._examples_raw = {}              # Contains steno rule IDs mapped to dicts of example translations.
+        self._examples_cache = {}            # Cache of example search data for each rule ID and mode.
 
     def _compile_data(self, translations:TranslationsDict, mode_strokes:bool) -> SearchData:
         """ Compile string search data for <translations> in the correct direction for <mode_strokes>. """
         if mode_strokes:
             d = forward_multidict(translations)
+            strip_chars = self._strip_strokes
         else:
             d = reverse_multidict(translations)
-        return _to_search_data(d, self._strip_chars)
+            strip_chars = self._strip_text
+        index = StripCaseIndex(strip_chars)
+        index.update(d)
+        d.update(_SENTINEL_MAP)
+        return (d, index)
 
     def set_translations(self, translations:TranslationsDict) -> None:
         """ Create new translation search data from the <translations> mapping. """
@@ -67,12 +64,23 @@ class SearchEngine:
             self._examples_cache[key] = self._compile_data(translations, mode_strokes)
         return self._examples_cache[key]
 
+    def lookup(self, pattern:str, *, mode_strokes=False) -> MatchTuple:
+        """ Perform an exact lookup for <pattern>, then a similar key lookup if nothing was found.
+            <mode_strokes> - If True, look up strokes instead of translations. """
+        d, index = self._get_translation_data(mode_strokes)
+        if pattern in d:
+            return d[pattern]
+        matches = []
+        for k in index.get_similar_keys(pattern):
+            matches += d[k]
+        return tuple(matches)
+
     def search(self, pattern:str, count=None, *, mode_strokes=False, mode_regex=False) -> MatchDict:
-        """ Perform a translations search for <pattern>. Return a special result if there's a regex syntax error.
+        """ Perform a detailed search for <pattern>. Unmatched keys in a result are sentinels with special behavior.
             If there is an index delimiter, search for rule examples instead. Only exact matches will work there.
             <count>        - Maximum number of matches returned. If None, there is no limit.
             <mode_strokes> - If True, search for strokes instead of translations.
-            <mode_regex>   - If True, do a regular expression search instead. """
+            <mode_regex>   - If True, do a regular expression search instead of a prefix search. """
         if not pattern.strip():
             return {}
         if INDEX_DELIM in pattern:
